#ifndef __NACC_PARSER_H
#define __NACC_PARSER_H

#include <stdint.h>  // for uint8_t
#include <vdr/ringbuffer.h>
#include "manager.h"
#include "interpreter.h"


class CCManager;


///////////////////////////////////////////////////////////////////////////////


class CCParser : public cThread 
{
public:
  CCParser(CCManager* manager, Interpreter* interpreter);
  virtual ~CCParser();
  void Put(const uint8_t* data, int len);
  
protected:
  virtual void Action(void);
  
private:
  void ProcessUserData(uint8_t* data, int length);
  void ProcessDishCC(uint8_t* data, int length);
  void ProcessATSCCC(uint8_t* data, int length);
  void Store(uint8_t* data, int length);
  
  cRingBufferLinear* inBuffer;
  
  uint8_t fBuff[64];
  int fPos;
  
  uint8_t dtvBuffer[64]; // Usually only need 4, but just to be safe...
  int dtvPos;
  uint8_t prevPictType;

  int unknownUserData;
  
  CCManager* manager;
  Interpreter* interpreter;
  
  static const uint8_t startUserDataHeader[];
  static const uint8_t endUserDataHeader[];
  static const uint8_t dishDVBHeader[];
  static const uint8_t ATSCHeader[];
  static const uint8_t r4000DVBHeader[];
  static const uint8_t r5000DVBHeader[];
  static const uint8_t dvdHeader[];	
};


///////////////////////////////////////////////////////////////////////////////


#endif //__NACC_PARSER_H
