#include "receiver.h"


///////////////////////////////////////////////////////////////////////////////


#if VDRVERSNUM >= 10500
CCReceiver::CCReceiver(CCParser* p, tChannelID ChannelID, int Vpid, int* Apid, int* Dpid)
:cReceiver(ChannelID, -1, Vpid, Apid, Dpid, NULL)
#else
CCReceiver::CCReceiver(CCParser* p, int Ca, int Vpid, int* Apid, int* Dpid)
:cReceiver(Ca, -1, Vpid, Apid, Dpid, NULL)
#endif
{
  parser = p;
  attachedDevice = NULL;
  pictureType = 0;
}


//-----------------------------------------------------------------------------
 
CCReceiver::~CCReceiver(void)
{
  Detach();
  dprint(L_DBG, "Receiver: Destructor.");
}

//-----------------------------------------------------------------------------

void CCReceiver::Attach(cDevice* device)
{
  if (!attachedDevice) 
  {
    attachedDevice = device;
    if (attachedDevice) 
    {
      attachedDevice->AttachReceiver(this);
      dprint(L_DBG, "Receiver: Attached.");
    }
  }
}


//-----------------------------------------------------------------------------

void CCReceiver::Detach(void)
{
  if (attachedDevice) 
  {
    attachedDevice->Detach(this);
    attachedDevice = NULL;
    dprint(L_DBG, "Receiver: Detached.");
  }
}


//-----------------------------------------------------------------------------

// Picture types:
//#define NO_PICTURE 0
//#define I_FRAME    1
//#define P_FRAME    2
//#define B_FRAME    3

// Adapted from VDR's remux.c
void CCReceiver::UpdatePictureType(uchar* Data, int Length)
{ 
  pictureType = 0;
  
  for (int i = 0; i < Length - 3; i++) {
    if (Data[i] == 0 && Data[i + 1] == 0 && Data[i + 2] == 1) 
    {
      uint8_t PesPayloadOffset = 6 + 3 + Data[8];
      const uchar *p = Data + i + PesPayloadOffset + 2;
      const uchar *pLimit = Data + i + Length - 3;
        
      while (p < pLimit && (p = (const uchar *)memchr(p, 0x01, pLimit - p))) 
      {
        if (!p[-2] && !p[-1]) 
        { // found 0x000001
          if (p[1] == 0x00) //SC_PICTURE
          {
            pictureType = (p[3] >> 3) & 0x07;
            return;
          }
            p += 4; // continue scanning after 0x01ssxxyy
        }
        else
          p += 3; // continue scanning after 0x01xxyy
      }
    }
  }
}


//-----------------------------------------------------------------------------
#if 0
uint64_t GetPTS(uchar* data)
{
  uint64_t pts = 0;
  uint8_t adaptation = (data[3] & 0x30) >> 4;
  uint8_t offset = 0;
  if (adaptation == 2 || adaptation == 3) // Adaptation field
    offset = data[4];
  
  uint8_t* d = data + 4 + offset;
 
  if (adaptation == 1 || adaptation == 3) // Payload
  {
    if (d[0]==0 && d[1]==0 && d[2]==1)
    {
      uint8_t pts_dts_flag = (d[7] & 0xC0) >> 6;
      if (pts_dts_flag == 2 || pts_dts_flag == 3)
      {
        pts = (((uint64_t)(d[9]  & 0x0E)) << 30) | (((uint64_t)d[10]) << 22) | 
              (((uint64_t)(d[11] & 0xFE)) << 15) | (((uint64_t)d[12]) << 7) | ((uint64_t)(d[13] & 0xFE));              
      }
    }
  }
  
  return pts;
}
#endif

//-----------------------------------------------------------------------------

#define TS_SIZE       188
#define PAYLOAD       0x10
#define ADAPT_FIELD   0x20
#define PAY_START     0x40
#define PTS_DTS_FLAGS 0xC0


void CCReceiver::Receive(uchar* Data, int Length)
{
  if (Length != TS_SIZE || Data[0] != 0x47)
    return; 
  
  if (!(Data[3] & PAYLOAD))
    return;
    
  uint8_t offset = 0;
  if (Data[3] & ADAPT_FIELD)
      offset = Data[4] + 1;
      
  if (Data[1] & PAY_START) 
  {
    uint8_t* sb = Data + 4 + offset;
    if (sb[7] & PTS_DTS_FLAGS) 
    { 
      uint8_t* pay = sb + sb[8] + 9;
      int l = TS_SIZE - 13 - offset - sb[8];
      if (l>0) 
      {
        for (int i=0; i<l; i++)
          if (pay[i]==0 && pay[i+1]==0 && pay[i+2]==1)
          {
            pay += i;
            l -= i;  
            break;        
          }
        
        bool gotATSC = false;
        
        for (int i=0; i<l-4; i++)
          if (pay[i]==0x47 && pay[i+1]==0x41 && pay[i+2]==0x39 && pay[i+3]==0x34) 
          {
            // For atsc/dtv cc we need to know the frame type
            UpdatePictureType(Data, Length);
            gotATSC = true;
            pay[i+4] = (pay[i+4]&0x07) | (pictureType << 3);

            //uint64_t pts = GetPTS(Data);
            //fprintf(stderr, "FT: %d PTS: %lld\n", pictureType, pts);

            break;
          }  
        
        //uint16_t pid = ((Data[1] & 0x1F) << 8) | Data[2];
        //fprintf(stderr, "PID: %d\n", pid);
       
        if (!gotATSC || pictureType != 0)
          parser->Put(pay, l);           
      }
    }
  }  
}


///////////////////////////////////////////////////////////////////////////////

